/*
 * Copyright (c) 2001-2006, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: ForwardAction.java 2494 2006-04-15 21:15:08Z jmettraux $
 */

//
// ForwardAction.java
//
// jmettraux@openwfe.org
//
// generated with 
// jtmpl 1.0.04 31.10.2002 John Mettraux (jmettraux@openwfe.org)
//

package openwfe.org.wlactions;

import javax.security.auth.Subject;

import openwfe.org.ApplicationContext;
import openwfe.org.engine.workitem.InFlowWorkItem;
import openwfe.org.engine.dispatch.DispatchingException;
import openwfe.org.engine.expressions.FlowExpressionId;
import openwfe.org.engine.participants.Participant;
import openwfe.org.engine.participants.ParticipantMap;
import openwfe.org.worklist.store.StoreException;


/**
 * The action of saving a (potentially) modified WI in a store
 *
 * <p><font size=2>CVS Info :
 * <br>$Author: jmettraux $
 * <br>$Date: 2006-04-15 23:15:08 +0200 (Sat, 15 Apr 2006) $
 * <br>$Id: ForwardAction.java 2494 2006-04-15 21:15:08Z jmettraux $ </font>
 *
 * @author jmettraux@openwfe.org
 */
public class ForwardAction

    extends WorkitemAction

{

    private final static org.apache.log4j.Logger log = org.apache.log4j.Logger
        .getLogger(ForwardAction.class.getName());

    //
    // FIELDS

    //
    // CONSTRUCTORS

    public ForwardAction 
        (final ApplicationContext worklistContext, 
         final Subject subject, 
         final String storeName,
         final InFlowWorkItem item)
    throws
        StoreException
    {
        super(worklistContext, subject, storeName, item);
    }

    //
    // METHODS

    public Object run () 
        throws StoreException, DispatchingException
    {
        //
        // make sure user has a lock on workitem before dispatching...
        
        if ( ! getWorkItemStore()
                .hasLockOn(getSubject(), getWorkitem().getLastExpressionId()))
        {
            throw new StoreException
                ("No lock on workitem, cannot forward it");
        }

        //
        // save workitem
        
        getWorkItemStore().save(getSubject(), getWorkitem());

        //
        // find participant and dispatch
        
        final ParticipantMap pMap = openwfe.org.engine.Definitions
            .getParticipantMap(getContext());

        // the 'participant' here is in fact, the engine hosting the flow the
        // workitem comes from
        //
        String engineId = getWorkitem().getLastExpressionId().getEngineId();

        log.debug("run() has to forward workitem to engine '"+engineId+"'");

        Participant p = pMap.get(engineId);

        if (p == null)
        {
            throw new DispatchingException 
                ("Cannot forward workitem to unknown engine/participant '"+
                 engineId+"'");
        }

        //
        // touch workitem
        
        getWorkitem().touch();

        //
        // dispatch

        p.dispatch(getContext(), getWorkitem());
        
        log.debug("run() successfully dispatched workitem to engine");

        //
        // remove workitem from store
        // (this happens only if the dispatching returned successfully)

        getWorkItemStore()
            .remove(getSubject(), getWorkitem().getLastExpressionId());

        log.debug("run() successfully removed workitem from store");

        return null;
    }

}
