/*
 * Copyright (c) 2001-2006, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: WorkSession.java 2746 2006-06-06 20:23:02Z jmettraux $
 */

//
// WorkSession.java
//
// jmettraux@openwfe.org
//
// generated with 
// jtmpl 1.0.04 31.10.2002 John Mettraux (jmettraux@openwfe.org)
//

package openwfe.org.worklist;

import java.rmi.RemoteException;
import javax.security.auth.Subject;
import openwfe.org.OpenWfeException;
import openwfe.org.engine.workitem.LaunchItem;
import openwfe.org.engine.workitem.InFlowWorkItem;
import openwfe.org.engine.expressions.FlowExpressionId;


/**
 * This interface details methods for interacting with a worklist.
 * The main implementation is openwfe.org.worklist.rmi.WorkSessionImpl
 *
 * <p><font size=2>CVS Info :
 * <br>$Author: jmettraux $
 * <br>$Date: 2006-06-06 22:23:02 +0200 (Tue, 06 Jun 2006) $
 * <br>$Id: WorkSession.java 2746 2006-06-06 20:23:02Z jmettraux $ </font>
 *
 * @author john.mettraux@openwfe.org
 */
public interface WorkSession

    extends java.rmi.Remote

{

    /*
     * There is already an init method to implement in RemoteSession...
     * No need for another one here.
     *
    public void init (ApplicationContext context, Subject s)
        throws RemoteException;
     *
     */

    public Subject getSubject ()
        throws RemoteException;

    /**
     * Lists the store names this worklist manages
     */
    public java.util.List getStoreNames ()
        throws RemoteException;

    /**
     * Returns the count of workitems in the given store
     */
    public int countWorkItems (String storeName)
        throws RemoteException, WorkListException;

    /**
     * Returns the flow expression ids of all the workitems belonging
     * to the given workflow instance.
     * This workflowInstanceId is the id given as the return value
     * of the launch() method
     */
    public java.util.List findFlowInstance
        (String storeName, String workflowInstanceId)
    throws 
        RemoteException, WorkListException;

    /**
     * Returns the headers of the workitems in the store. You can set
     * a limit if you don't want to much headers to come back.
     */
    public java.util.List getHeaders 
        (String storeName, int limit)
    throws 
        RemoteException, WorkListException;

    /**
     * Same as getHeaders, but you can supply a headerComparator which will
     * sort headers.
     */
    public java.util.List getHeaders 
        (String storeName, int limit, java.util.Comparator headerComparator)
    throws 
        RemoteException, WorkListException;

    /**
     * Retrieves a workitem from a store
     */
    public InFlowWorkItem get (String storeName, FlowExpressionId fei)
        throws RemoteException, WorkListException;

    /**
     * Retrieves a workitem from a store
     */
    public InFlowWorkItem get (String storeName, String id)
        throws RemoteException, WorkListException;

    /**
     * Retrieves a workitem from a store and ensure that you will be the
     * only able to tinker with it.
     */
    public InFlowWorkItem getAndLock (String storeName, FlowExpressionId fei)
        throws RemoteException, WorkListException;

    /**
     * Retrieves a workitem from a store and ensure that you will be the
     * only able to tinker with it.
     */
    public InFlowWorkItem getAndLock (String storeName, String id)
        throws RemoteException, WorkListException;

    /**
     * Unlocks a workitem (no modifications are kept).
     */
    public void release (String storeName, InFlowWorkItem item)
        throws RemoteException, WorkListException;

    /**
     * Saves a workitem (the modifications you made to it).
     */
    public void save (String storeName, InFlowWorkItem item)
        throws RemoteException, WorkListException;

    /**
     * Gives the workitem back to the worklist so that it can proceed
     * in its flow ; changes to the workitem are kept.
     * (This is the equivalent of the 'proceed' command in the webclient)
     */
    public void forward (String storeName, InFlowWorkItem item)
        throws RemoteException, OpenWfeException;

    /**
     * Returns a list of flows the user is entitled to launch
     */
    public java.util.List getLaunchables ()
        throws RemoteException, WorkListException;

    /**
     * Launches a flow in a given engine, with a launchitem containing info
     * about which flow to launch
     *
     * @return the workflow instance id of the newly created flow
     */
    public String launch (String engineId, LaunchItem item)
        throws RemoteException, OpenWfeException;

    /**
     * Delegates a workitem from one store to the other in the same worklist.
     */
    public void delegate 
        (String storeName, InFlowWorkItem item, String targetStoreName)
    throws RemoteException, WorkListException;

    /**
     * Another type of delegation : when you delegate directly to a participant.
     * This method lists the participant the user/client has the right to 
     * delegate to (with the next method).
     */
    public java.util.List getParticipantsForDelegation ()
        throws RemoteException, WorkListException;

    /**
     * The method to call for direct delegation.
     */
    public void delegateToParticipant
        (String storeName, InFlowWorkItem item, String participantName)
    throws 
        RemoteException, WorkListException;

    /**
     * Closes this worksession (releases it).
     */
    public void close ()
        throws RemoteException;

    //
    // info methods

    /**
     * Checks the right you currently have (as owner of a worksession) on a 
     * given storeName. 
     * Action names may be read, write, delete or delegate.
     */
    public boolean checkPermission (String storeName, String action)
        throws RemoteException, WorkListException;

}
